module.exports = {
  parser: 'babel-eslint',
  extends: [
    'plugin:react/recommended',
    'airbnb',
  ],
  plugins: [
    'import',
    'react',
  ],
  parserOptions: {
    ecmaVersion: 2017,
    sourceType: 'module',
    ecmaFeatures: {
      experimentalObjectRestSpread: true,
    },
  },
  rules: {
    'strict': 'off',
    'no-unused-vars': ['warn', { varsIgnorePattern: 'React', ignoreRestSiblings: false }],
    'class-methods-use-this': ['warn', { exceptMethods: [
        'componentDidCatch',
        'componentDidMount',
        'componentDidUpdate',
        'componentWillMount',
        'componentWillReceiveProps',
        'componentWillUnmount',
        'componentWillUpdate',
        'render',
        'shouldComponentUpdate',
      ]}],

    'comma-dangle': ['error', {
      arrays: 'always-multiline',
      objects: 'always-multiline',
      imports: 'always-multiline',
      exports: 'always-multiline',
      functions: 'always-multiline',
    }],

    'no-restricted-syntax': [
      'error',
      {
        selector: 'ForInStatement',
        message: 'for..in loops iterate over the entire prototype chain, which is virtually never what you want. Use Object.{keys,values,entries}, and iterate over the resulting array.',
      },
      {
        selector: 'ForOfStatement',
        message: 'iterators/generators require regenerator-runtime, which is too heavyweight for this guide to allow them. Separately, loops should be avoided in favor of array iterations.',
      },
      {
        selector: 'LabeledStatement',
        message: 'Labels are a form of GOTO; using them makes code confusing and hard to maintain and understand.',
      },
      {
        selector: 'WithStatement',
        message: '`with` is disallowed in strict mode because it makes code impossible to predict and optimize.',
      },
    ],

    'no-await-in-loop': 'error',

    'import/no-extraneous-dependencies': 'off',

    'react/jsx-filename-extension': ['error', { extensions: ['.js', '.jsx'] }],
    'react/prop-types': ['warn', {
      ignore: ['classes', 'theme', 'sheet', 'sheets'],
    }],
    'react/no-access-state-in-setstate': 'warn',
    'react/forbid-foreign-prop-types': 'warn',
    'react/jsx-max-depth': ['warn', { max: 5 }],

    'jsx-a11y/interactive-supports-focus': 'off',
    'jsx-a11y/href-no-hash': 'off',
    'jsx-a11y/click-events-have-key-events': 'off',
    'vars-on-top': 'off',
    semi: ['error', 'never'],
    'no-confusing-arrow': 'off',
    'object-property-newline': ['error', {
      allowAllPropertiesOnSameLine: true,
    }],

    'object-curly-newline': ['warn', {
      ObjectExpression: { multiline: true, consistent: true },
      ObjectPattern: { multiline: true, consistent: true },
      ImportDeclaration: { multiline: true, consistent: true },
      ExportDeclaration: { multiline: true, consistent: true },
    }],
    'object-curly-spacing': ['error', 'always'],

    'arrow-parens': ['warn', 'always'],

    'id-match': ['error', '^(([A-Za-z0-9]+){2,})|([A-Z][A-Z_0-9]+)$', {
      properties: false,
      onlyDeclarations: true,
    }],

    'no-magic-numbers': ['warn', {
      ignore: [1, 0, -1],
      ignoreArrayIndexes: true,
      enforceConst: true,
      detectObjects: false,
    }],

    'operator-linebreak': ['error', 'after', {
      overrides: {
        '?': 'before',
        ':': 'before',
        '&&': 'before',
        '+': 'before',
        '||': 'before',
      } },
    ],
    'brace-style': ['error', 'stroustrup'],
    'quote-props': ['error', 'as-needed'],
    'no-plusplus': 'off',
    'lines-between-class-members': ['error', 'always', { exceptAfterSingleLine: true }],
    'implicit-arrow-linebreak': ['error', 'beside'],

    'padding-line-between-statements': [
      'error',
      { blankLine: 'always', prev: ['const', 'let', 'var'], next: '*' },
      { blankLine: 'any', prev: ['const', 'let', 'var'], next: ['const', 'let', 'var'] },
    ],
    'import/no-absolute-path': 'error',
    'import/no-unresolved': 'off',
    'import/extensions': ['error', 'never',
      [
        'json', 'json5',
        'less', 'css', 'scss', 'sass', 'styl',
        'jpeg', 'jpg', 'png', 'svg', 'bmp', 'gif',
      ].reduce((obj, ext) => Object.assign(obj, { [ext]: 'always' }), {}),
    ],

    'import/no-default-export': 'error',
    'import/no-self-import': 'error',
    'import/no-cycle': 'error',

    'import/order': ['warn', {
      groups: [
        ['builtin', 'external'],
        ['internal', 'parent'],
        ['sibling', 'index'],
      ],
      'newlines-between': 'ignore',
    }],

    'import/newline-after-import': ['warn', { count: 2 }],
    'import/prefer-default-export': 'off',
    'import/no-named-as-default': 'off',
  },
}